@@ -26,10 +26,38 @@ struct fieldinfo;
 
 struct parsedb_state {
 	enum parsedbflags flags;
+	struct pkginfo *pkg;
+	struct pkgbin *pkgbin;
+	char *data;
+	char *dataptr;
+	char *endptr;
 	const char *filename;
 	int lno;
 };
 
+#define parse_EOF(ps)		((ps)->dataptr >= (ps)->endptr)
+#define parse_getc(ps)		*(ps)->dataptr++
+#define parse_ungetc(c, ps)	(ps)->dataptr--
+
+struct field_state {
+	const char *fieldstart;
+	const char *valuestart;
+	struct varbuf value;
+	int fieldlen;
+	int valuelen;
+	int *fieldencountered;
+};
+
+void parse_open(struct parsedb_state *ps, const char *filename,
+                enum parsedbflags flags);
+void parse_close(struct parsedb_state *ps);
+
+typedef void parse_field_func(struct parsedb_state *ps, struct field_state *fs,
+                              void *parse_obj);
+
+bool parse_stanza(struct parsedb_state *ps, struct field_state *fs,
+                  parse_field_func *parse_field, void *parse_obj);
+
 #define PKGIFPOFF(f) (offsetof(struct pkgbin, f))
 #define PKGPFIELD(pifp,of,type) (*(type*)((char*)(pifp)+(of)))
 
@@ -64,19 +92,17 @@ struct fieldinfo {
   size_t integer;
 };
 
-void parse_db_version(struct parsedb_state *ps, const struct pkginfo *pkg,
+void parse_db_version(struct parsedb_state *ps,
                       struct versionrevision *version, const char *value,
-                      const char *fmt, ...) DPKG_ATTR_PRINTF(5);
+                      const char *fmt, ...) DPKG_ATTR_PRINTF(4);
 
-void parse_error(struct parsedb_state *ps, const struct pkginfo *pigp,
-                 const char *fmt, ...) DPKG_ATTR_NORET DPKG_ATTR_PRINTF(3);
-void parse_warn(struct parsedb_state *ps, const struct pkginfo *pigp,
-                const char *fmt, ...) DPKG_ATTR_PRINTF(3);
+void parse_error(struct parsedb_state *ps, const char *fmt, ...)
+	DPKG_ATTR_NORET DPKG_ATTR_PRINTF(2);
+void parse_warn(struct parsedb_state *ps, const char *fmt, ...)
+	DPKG_ATTR_PRINTF(2);
 void parse_must_have_field(struct parsedb_state *ps,
-                           const struct pkginfo *pigp,
                            const char *value, const char *what);
 void parse_ensure_have_field(struct parsedb_state *ps,
-                             const struct pkginfo *pigp,
                              const char **value, const char *what);
 
 #define MSDOS_EOF_CHAR '\032' /* ^Z */
@@ -87,6 +113,5 @@ struct nickname {
 };
 
 extern const struct fieldinfo fieldinfos[];
-extern const struct nickname nicknames[];
 
 #endif /* LIBDPKG_PARSEDUMP_H */
