@@ -42,8 +42,10 @@
 #include <dpkg/i18n.h>
 #include <dpkg/dpkg.h>
 #include <dpkg/dpkg-db.h>
+#include <dpkg/string.h>
 #include <dpkg/buffer.h>
 #include <dpkg/file.h>
+#include <dpkg/path.h>
 #include <dpkg/subproc.h>
 #include <dpkg/command.h>
 #include <dpkg/triglib.h>
@@ -354,6 +356,7 @@ deferred_configure(struct pkginfo *pkg)
 	                           NULL);
 
 	pkg->eflag = eflag_ok;
+	pkg->trigpend_head = NULL;
 	post_postinst_tasks(pkg, stat_installed);
 }
 
@@ -373,7 +376,7 @@ conffderef(struct pkginfo *pkg, struct varbuf *result, const char *in)
 {
 	static struct varbuf target = VARBUF_INIT;
 	struct stat stab;
-	int r;
+	ssize_t r;
 	int loopprotect;
 
 	varbuf_reset(result);
@@ -416,13 +419,17 @@ conffderef(struct pkginfo *pkg, struct varbuf *result, const char *in)
 				          " (= '%s'): %s"),
 				        pkg->name, in, result->buf, strerror(errno));
 				return -1;
+			} else if (r != stab.st_size) {
+				warning(_("symbolic link '%.250s' size has "
+				          "changed from %jd to %zd"),
+				        result->buf, stab.st_size, r);
+				return -1;
 			}
-			assert(r == stab.st_size); /* XXX: debug */
 			varbuf_trunc(&target, r);
 			varbuf_end_str(&target);
 
 			debug(dbg_conffdetail,
-			      "conffderef readlink gave %d, '%s'",
+			      "conffderef readlink gave %zd, '%s'",
 			      r, target.buf);
 
 			if (target.buf[0] == '/') {
@@ -510,7 +517,7 @@ showdiff(const char *old, const char *new)
 			pager = DEFAULTPAGER;
 
 		sprintf(cmdbuf, DIFF " -Nu %.250s %.250s | %.250s",
-		        old, new, pager);
+		        str_quote_meta(old), str_quote_meta(new), pager);
 
 		command_shell(cmdbuf, _("conffile difference visualizer"));
 	}
@@ -653,9 +660,7 @@ promptconfaction(struct pkginfo *pkg, const char *cfgfile,
 		else if (what & cfof_install)
 			fprintf(stderr, _(" The default action is to install the new version.\n"));
 
-		s = strrchr(cfgfile, '/');
-		if (!s || !*++s)
-			s = cfgfile;
+		s = path_basename(cfgfile);
 		fprintf(stderr, "*** %s (Y/I/N/O/D/Z) %s ? ",
 		        s,
 		        (what & cfof_keep) ? _("[default=N]") :
