@@ -4,7 +4,7 @@
  *
  * Copyright © 1995,1996 Ian Jackson <ian@chiark.greenend.org.uk>
  * Copyright © 2000,2001 Wichert Akkerman <wakkerma@debian.org>
- * Copyright © 2006-2009 Guillem Jover <guillem@debian.org>
+ * Copyright © 2006-2011 Guillem Jover <guillem@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,9 +46,10 @@
 #include <dpkg/pkg-format.h>
 #include <dpkg/pkg-show.h>
 #include <dpkg/path.h>
-#include <dpkg/myopt.h>
+#include <dpkg/options.h>
 
 #include "filesdb.h"
+#include "infodb.h"
 #include "main.h"
 
 static const char* showformat		= "${Package}\t${Version}\n";
@@ -64,68 +65,128 @@ static int getwidth(void) {
   else if (!isatty(1))
     return -1;
   else {
+    res = 80;
+
     if ((fd=open("/dev/tty",O_RDONLY))!=-1) {
-      if (ioctl(fd, TIOCGWINSZ, &ws)==-1)
-	ws.ws_col=80;
+      if (ioctl(fd, TIOCGWINSZ, &ws) == 0)
+        res = ws.ws_col;
       close(fd);
     }
-    return ws.ws_col;
+
+    return res;
   }
 }
 
+struct list_format {
+  bool head;
+  int nw, vw, dw;
+  char format[80];
+};
+
 static void
-list1package(struct pkginfo *pkg, bool *head, struct pkg_array *array)
+list_format_init(struct list_format *fmt, struct pkg_array *array)
 {
-  int i,l,w;
-  static int nw,vw,dw;
-  const char *pdesc;
-  static char format[80]   = "";
-    
-  if (format[0] == '\0') {
-    w=getwidth();
-    if (w == -1) {
-      nw=14, vw=14, dw=44;
-      for (i = 0; i < array->n_pkgs; i++) {
-	int plen, vlen, dlen;
+  int w;
 
-	plen = strlen(array->pkgs[i]->name);
-	vlen = strlen(versiondescribe(&array->pkgs[i]->installed.version,
-	                              vdew_nonambig));
-	pkg_summary(array->pkgs[i], &dlen);
+  if (fmt->format[0] != '\0')
+    return;
 
-	if (plen > nw) nw = plen;
-	if (vlen > vw) vw = vlen;
-	if (dlen > dw) dw = dlen;
-      }
-    } else {
-      w-=80;
-      if (w<0) w=0;		/* lets not try to deal with terminals that are too small */
-      w>>=2;		/* halve that so we can add that to the both the name and description */
-      nw=(14+w);		/* name width */
-      vw=(14+w);		/* version width */
-      dw=(44+(2*w));	/* description width */
+  w = getwidth();
+  if (w == -1) {
+    int i;
+
+    fmt->nw = 14;
+    fmt->vw = 14;
+    fmt->dw = 44;
+
+    for (i = 0; i < array->n_pkgs; i++) {
+      int plen, vlen, dlen;
+
+      plen = strlen(array->pkgs[i]->name);
+      vlen = strlen(versiondescribe(&array->pkgs[i]->installed.version,
+                                    vdew_nonambig));
+      pkg_summary(array->pkgs[i], &dlen);
+
+      if (plen > fmt->nw)
+        fmt->nw = plen;
+      if (vlen > fmt->vw)
+        fmt->vw = vlen;
+      if (dlen > fmt->dw)
+        fmt->dw = dlen;
     }
-    sprintf(format,"%%c%%c%%c %%-%d.%ds %%-%d.%ds %%.*s\n", nw, nw, vw, vw);
+  } else {
+    w -= 80;
+    /* Let's not try to deal with terminals that are too small. */
+    if (w < 0)
+      w = 0;
+    /* Halve that so we can add it to both the name and description. */
+    w >>= 2;
+    /* Name width. */
+    fmt->nw = (14 + w);
+    /* Version width. */
+    fmt->vw = (14 + w);
+    /* Description width. */
+    fmt->dw = (44 + (2 * w));
   }
+  sprintf(fmt->format, "%%c%%c%%c %%-%d.%ds %%-%d.%ds %%.*s\n",
+          fmt->nw, fmt->nw, fmt->vw, fmt->vw);
+}
 
-  if (!*head) {
-    fputs(_("\
+static void
+list_format_print_header(struct list_format *fmt)
+{
+  int l;
+
+  if (fmt->head)
+    return;
+
+  /* TRANSLATORS: This is the header that appears on 'dpkg-query -l'. The
+   * string should remain under 80 characters. The uppercase letters in
+   * the state values denote the abbreviated letter that will appear on
+   * the first three columns, which should ideally match the English one
+   * (e.g. Remove → supRimeix), see dpkg-query(1) for further details. The
+   * translated message can use additional lines if needed. */
+  fputs(_("\
 Desired=Unknown/Install/Remove/Purge/Hold\n\
 | Status=Not/Inst/Conf-files/Unpacked/halF-conf/Half-inst/trig-aWait/Trig-pend\n\
 |/ Err?=(none)/Reinst-required (Status,Err: uppercase=bad)\n"), stdout);
-    printf(format,'|','|','/', _("Name"), _("Version"), 40, _("Description"));
-    printf("+++-");					/* status */
-    for (l=0;l<nw;l++) printf("="); printf("-");	/* packagename */
-    for (l=0;l<vw;l++) printf("="); printf("-");	/* version */
-    for (l=0;l<dw;l++) printf("="); 			/* description */
-    printf("\n");
-    *head = true;
-  }
+  printf(fmt->format, '|', '|', '/', _("Name"), _("Version"), 40,
+         _("Description"));
+
+  /* Status */
+  printf("+++-");
+
+ /* Package name. */
+  for (l = 0; l < fmt->nw; l++)
+    printf("=");
+  printf("-");
+
+  /* Version. */
+  for (l = 0; l < fmt->vw; l++)
+    printf("=");
+  printf("-");
+
+  /* Description. */
+  for (l = 0; l < fmt->dw; l++)
+    printf("=");
+  printf("\n");
+
+  fmt->head = true;
+}
+
+static void
+list1package(struct pkginfo *pkg, struct list_format *fmt, struct pkg_array *array)
+{
+  int l;
+  const char *pdesc;
+
+  list_format_init(fmt, array);
+  list_format_print_header(fmt);
 
   pdesc = pkg_summary(pkg, &l);
-  l = min(l, dw);
+  l = min(l, fmt->dw);
 
-  printf(format,
+  printf(fmt->format,
          "uihrp"[pkg->want],
          "ncHUFWti"[pkg->status],
          " R"[pkg->eflag],
@@ -141,20 +202,24 @@ listpackages(const char *const *argv)
   struct pkginfo *pkg;
   int i;
   int failures = 0;
-  bool head;
+  struct list_format fmt;
 
-  modstatdb_init(admindir,msdbrw_readonly);
+  if (!*argv)
+    modstatdb_open(msdbrw_readonly);
+  else
+    modstatdb_open(msdbrw_readonly | msdbrw_available_readonly);
 
   pkg_array_init_from_db(&array);
   pkg_array_sort(&array, pkg_sorter_by_name);
 
-  head = false;
+  fmt.head = false;
+  fmt.format[0] = '\0';
 
   if (!*argv) {
     for (i = 0; i < array.n_pkgs; i++) {
       pkg = array.pkgs[i];
       if (pkg->status == stat_notinstalled) continue;
-      list1package(pkg, &head, &array);
+      list1package(pkg, &fmt, &array);
     }
   } else {
     int argc, ip, *found;
@@ -167,7 +232,7 @@ listpackages(const char *const *argv)
       pkg = array.pkgs[i];
       for (ip = 0; ip < argc; ip++) {
         if (!fnmatch(argv[ip], pkg->name, 0)) {
-          list1package(pkg, &head, &array);
+          list1package(pkg, &fmt, &array);
           found[ip]++;
           break;
         }
@@ -239,36 +304,35 @@ searchfiles(const char *const *argv)
   int failures = 0;
   struct varbuf path = VARBUF_INIT;
   static struct varbuf vb;
-  
+
   if (!*argv)
     badusage(_("--search needs at least one file name pattern argument"));
 
-  modstatdb_init(admindir,msdbrw_readonly|msdbrw_noavail);
+  modstatdb_open(msdbrw_readonly);
   ensure_allinstfiles_available_quiet();
   ensure_diversions();
 
   while ((thisarg = *argv++) != NULL) {
     found= 0;
 
-    /* Trim trailing slash and slash dot from the argument if it's
-     * not a pattern, just a path.
-     */
+    /* Trim trailing ‘/’ and ‘/.’ from the argument if it's
+     * not a pattern, just a path. */
     if (!strpbrk(thisarg, "*[?\\")) {
-      varbufreset(&path);
-      varbufaddstr(&path, thisarg);
-      varbufaddc(&path, '\0');
+      varbuf_reset(&path);
+      varbuf_add_str(&path, thisarg);
+      varbuf_end_str(&path);
 
-      varbuf_trunc(&path, path_rtrim_slash_slashdot(path.buf));
+      varbuf_trunc(&path, path_trim_slash_slashdot(path.buf));
 
       thisarg = path.buf;
     }
 
     if (!strchr("*[?/",*thisarg)) {
-      varbufreset(&vb);
-      varbufaddc(&vb,'*');
-      varbufaddstr(&vb,thisarg);
-      varbufaddc(&vb,'*');
-      varbufaddc(&vb,0);
+      varbuf_reset(&vb);
+      varbuf_add_char(&vb, '*');
+      varbuf_add_str(&vb, thisarg);
+      varbuf_add_char(&vb, '*');
+      varbuf_end_str(&vb);
       thisarg= vb.buf;
     }
     if (!strpbrk(thisarg, "*[?\\")) {
@@ -283,7 +347,8 @@ searchfiles(const char *const *argv)
       iterfileend(it);
     }
     if (!found) {
-      fprintf(stderr,_("dpkg: %s not found.\n"),thisarg);
+      fprintf(stderr, _("%s: no path found matching pattern %s.\n"),
+              dpkg_get_progname(), thisarg);
       failures++;
       m_output(stderr, _("<standard error>"));
     } else {
@@ -305,50 +370,46 @@ enqperpackage(const char *const *argv)
   struct pkginfo *pkg;
   struct filenamenode *namenode;
   int failures = 0;
-  
+
   if (!*argv)
     badusage(_("--%s needs at least one package name argument"), cipaction->olong);
 
-  if (cipaction->arg==act_listfiles)
-    modstatdb_init(admindir,msdbrw_readonly|msdbrw_noavail);
-  else 
-    modstatdb_init(admindir,msdbrw_readonly);
+  if (cipaction->arg_int == act_printavail)
+    modstatdb_open(msdbrw_readonly | msdbrw_available_readonly);
+  else
+    modstatdb_open(msdbrw_readonly);
 
   while ((thisarg = *argv++) != NULL) {
-    pkg= findpackage(thisarg);
+    pkg = pkg_db_find(thisarg);
 
-    switch (cipaction->arg) {
-      
+    switch (cipaction->arg_int) {
     case act_status:
       if (pkg->status == stat_notinstalled &&
           pkg->priority == pri_unknown &&
           !(pkg->section && *pkg->section) &&
           !pkg->files &&
           pkg->want == want_unknown &&
-          !informative(pkg,&pkg->installed)) {
+          !pkg_is_informative(pkg, &pkg->installed)) {
         fprintf(stderr,_("Package `%s' is not installed and no info is available.\n"),pkg->name);
         failures++;
       } else {
         writerecord(stdout, _("<standard output>"), pkg, &pkg->installed);
       }
       break;
-
     case act_printavail:
-      if (!informative(pkg,&pkg->available)) {
+      if (!pkg_is_informative(pkg, &pkg->available)) {
         fprintf(stderr,_("Package `%s' is not available.\n"),pkg->name);
         failures++;
       } else {
         writerecord(stdout, _("<standard output>"), pkg, &pkg->available);
       }
       break;
-      
     case act_listfiles:
       switch (pkg->status) {
-      case stat_notinstalled: 
+      case stat_notinstalled:
         fprintf(stderr,_("Package `%s' is not installed.\n"),pkg->name);
         failures++;
         break;
-        
       default:
         ensure_packagefiles_available(pkg);
         ensure_diversions();
@@ -377,9 +438,8 @@ enqperpackage(const char *const *argv)
         break;
       }
       break;
-
     default:
-      internerr("unknown action '%d'", cipaction->arg);
+      internerr("unknown action '%d'", cipaction->arg_int);
     }
 
     if (*argv != NULL)
@@ -412,7 +472,10 @@ showpackages(const char *const *argv)
     return failures;
   }
 
-  modstatdb_init(admindir,msdbrw_readonly);
+  if (!*argv)
+    modstatdb_open(msdbrw_readonly);
+  else
+    modstatdb_open(msdbrw_readonly | msdbrw_available_readonly);
 
   pkg_array_init_from_db(&array);
   pkg_array_sort(&array, pkg_sorter_by_name);
@@ -461,83 +524,33 @@ showpackages(const char *const *argv)
   return failures;
 }
 
+static void
+pkg_infodb_print_filename(const char *filename, const char *filetype)
+{
+  /* Do not expose internal database files. */
+  if (strcmp(filetype, LISTFILE) == 0 ||
+      strcmp(filetype, CONFFILESFILE) == 0)
+    return;
+
+  if (strlen(filetype) > MAXCONTROLFILENAME)
+    return;
+
+  printf("%s\n", filename);
+}
+
 static void
 control_path_file(struct pkginfo *pkg, const char *control_file)
 {
   const char *control_path;
   struct stat st;
 
-  /* Do not expose internal database files. */
-  if (strcmp(control_file, LISTFILE) == 0 ||
-      strcmp(control_file, CONFFILESFILE) == 0)
-    return;
-
   control_path = pkgadminfile(pkg, control_file);
-
   if (stat(control_path, &st) < 0)
     return;
-
   if (!S_ISREG(st.st_mode))
     return;
 
-  printf("%s\n", control_path);
-}
-
-static void
-control_path_pkg(struct pkginfo *pkg)
-{
-  DIR *db_dir;
-  struct dirent *db_de;
-  struct varbuf db_path;
-  size_t db_path_len;
-
-  varbufinit(&db_path, 0);
-  varbufaddstr(&db_path, pkgadmindir());
-  db_path_len = db_path.used;
-  varbufaddc(&db_path, '\0');
-
-  db_dir = opendir(db_path.buf);
-  if (!db_dir)
-    ohshite(_("cannot read info directory"));
-
-  push_cleanup(cu_closedir, ~0, NULL, 0, 1, (void *)db_dir);
-  while ((db_de = readdir(db_dir)) != NULL) {
-    const char *p;
-
-    /* Ignore dotfiles, including ‘.’ and ‘..’. */
-    if (db_de->d_name[0] == '.')
-      continue;
-
-    /* Ignore anything odd. */
-    p = strrchr(db_de->d_name, '.');
-    if (!p)
-      continue;
-
-    /* Ignore files from other packages. */
-    if (strlen(pkg->name) != (size_t)(p - db_de->d_name) ||
-        strncmp(db_de->d_name, pkg->name, p - db_de->d_name))
-      continue;
-
-    /* Skip past the full stop. */
-    p++;
-
-    /* Do not expose internal database files. */
-    if (strcmp(p, LISTFILE) == 0 ||
-        strcmp(p, CONFFILESFILE) == 0)
-      continue;
-
-    if (strlen(p) > MAXCONTROLFILENAME)
-      continue;
-
-    varbuf_trunc(&db_path, db_path_len);
-    varbufaddstr(&db_path, db_de->d_name);
-    varbufaddc(&db_path, '\0');
-
-    printf("%s\n", db_path.buf);
-  }
-  pop_cleanup(ehflag_normaltidy); /* closedir */
-
-  varbuf_destroy(&db_path);
+  pkg_infodb_print_filename(control_path, control_file);
 }
 
 static int
@@ -565,16 +578,16 @@ control_path(const char *const *argv)
         badusage(_("control file contains %c"), *c);
   }
 
-  modstatdb_init(admindir, msdbrw_readonly | msdbrw_noavail);
+  modstatdb_open(msdbrw_readonly);
 
-  pkg = findpackage(pkg_name);
+  pkg = pkg_db_find(pkg_name);
   if (pkg->status == stat_notinstalled)
-    badusage(_("Package `%s' is not installed.\n"), pkg->name);
+    ohshit(_("Package `%s' is not installed.\n"), pkg->name);
 
   if (control_file)
     control_path_file(pkg, control_file);
   else
-    control_path_pkg(pkg);
+    pkg_infodb_foreach(pkg, pkg_infodb_print_filename);
 
   modstatdb_shutdown();
 
@@ -608,7 +621,7 @@ usage(const struct cmdinfo *ci, const char *value)
 "  -p|--print-avail <package> ...   Display available version details.\n"
 "  -L|--listfiles <package> ...     List files `owned' by package(s).\n"
 "  -l|--list [<pattern> ...]        List packages concisely.\n"
-"  -W|--show <pattern> ...          Show information on package(s).\n"
+"  -W|--show [<pattern> ...]        Show information on package(s).\n"
 "  -S|--search <pattern> ...        Find package(s) owning file(s).\n"
 "  -c|--control-path <package> [<file>]\n"
 "                                   Print path for package control file.\n"
@@ -639,30 +652,15 @@ usage(const struct cmdinfo *ci, const char *value)
   exit(0);
 }
 
-const char thisname[]= "dpkg-query";
-const char printforhelp[]= N_("Use --help for help about querying packages.");
+static const char printforhelp[] = N_(
+"Use --help for help about querying packages.");
 
-const struct cmdinfo *cipaction = NULL;
-
-const char *admindir= ADMINDIR;
-
-static void setaction(const struct cmdinfo *cip, const char *value) {
-  if (cipaction)
-    badusage(_("conflicting actions -%c (--%s) and -%c (--%s)"),
-             cip->oshort, cip->olong, cipaction->oshort, cipaction->olong);
-  cipaction= cip;
-}
+static const char *admindir;
 
+/* This table has both the action entries in it and the normal options.
+ * The action entries are made with the ACTION macro, as they all
+ * have a very similar structure. */
 static const struct cmdinfo cmdinfos[]= {
-  /* This table has both the action entries in it and the normal options.
-   * The action entries are made with the ACTION macro, as they all
-   * have a very similar structure.
-   */
-#define ACTION(longopt,shortopt,code,function) \
- { longopt, shortopt, 0, NULL, NULL, setaction, code, NULL, (voidfnp)function }
-#define OBSOLETE(longopt,shortopt) \
- { longopt, shortopt, 0, NULL, NULL, setobsolete, 0, NULL, NULL }
-
   ACTION( "listfiles",                      'L', act_listfiles,     enqperpackage   ),
   ACTION( "status",                         's', act_status,        enqperpackage   ),
   ACTION( "print-avail",                    'p', act_printavail,    enqperpackage   ),
@@ -679,25 +677,24 @@ static const struct cmdinfo cmdinfos[]= {
 };
 
 int main(int argc, const char *const *argv) {
-  jmp_buf ejbuf;
-  int (*actionfunction)(const char *const *argv);
   int ret;
 
   setlocale(LC_ALL, "");
   bindtextdomain(PACKAGE, LOCALEDIR);
   textdomain(PACKAGE);
 
-  standard_startup(&ejbuf);
-  myopt(&argv, cmdinfos);
+  dpkg_set_progname("dpkg-query");
+  standard_startup();
+  myopt(&argv, cmdinfos, printforhelp);
+
+  admindir = dpkg_db_set_dir(admindir);
 
   if (!cipaction) badusage(_("need an action option"));
 
   setvbuf(stdout, NULL, _IONBF, 0);
   filesdbinit();
 
-  actionfunction = (int (*)(const char *const *))cipaction->farg;
-
-  ret = actionfunction(argv);
+  ret = cipaction->action(argv);
 
   standard_shutdown();
 
